@@ -24,10 +24,10 @@ to add to this and submit patches to make it more complete.
 
 import os
 import pkgutil
-import shutil
 import sys
 
 from nuitka.containers.oset import OrderedSet
+from nuitka.freezer.IncludedEntryPoints import makeDllEntryPoint
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.PythonVersions import python_version
 from nuitka.utils.FileOperations import getFileContentByLine
@@ -35,8 +35,6 @@ from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.SharedLibraries import getPyWin32Dir, locateDLL
 from nuitka.utils.Utils import getOS, isWin32Windows
 
-_added_pywin32 = False
-
 
 class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
     plugin_name = "implicit-imports"
@@ -106,6 +104,11 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "cairo"
         elif full_name == "cairo._cairo":
             yield "gi._gobject"
+        elif full_name == "gi.overrides":
+            yield "gi.overrides.Gtk"
+            yield "gi.overrides.Gdk"
+            yield "gi.overrides.GLib"
+            yield "gi.overrides.GObject"
         elif full_name in ("Tkinter", "tkinter"):
             yield "_tkinter"
         elif full_name == "cryptography":
@@ -116,6 +119,8 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "_cffi_backend"
         elif full_name == "brotli._brotli":
             yield "_cffi_backend"
+        elif full_name == "ipcqueue":
+            yield "_cffi_backend"
         elif full_name == "_dbus_glib_bindings":
             yield "_dbus_bindings"
         elif full_name == "_mysql":
@@ -713,6 +718,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         elif full_name == "skimage.filters.rank":
             yield "skimage.filters.rank.bilateral_cy"
             yield "skimage.filters.rank.core_cy"
+            yield "skimage.filters.rank.core_cy_3d"
             yield "skimage.filters.rank.generic_cy"
             yield "skimage.filters.rank.percentile_cy"
 
@@ -871,6 +877,9 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         elif full_name == "sklearn.utils.sparsetools":
             yield "sklearn.utils.sparsetools._graph_validation"
             yield "sklearn.utils.sparsetools._graph_tools"
+
+        elif full_name == "sklearn.utils._hough_transform":
+            yield "skimage.draw"
         # end of scikit-learn imports -----------------------------------------
 
         elif full_name == "PIL._imagingtk":
@@ -1033,6 +1042,8 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "cytoolz.utils"
         elif full_name == "cytoolz.functoolz":
             yield "cytoolz._signatures"
+        elif full_name == "exchangelib":
+            yield "tzdata"
 
     def getImportsByFullname(self, full_name, module_filename):
         """Recursively create a set of imports for a fullname.
@@ -1258,22 +1269,22 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             uuid_dll_path = locateDLL("uuid")
 
             if uuid_dll_path is not None:
-                dist_dll_path = os.path.join(dist_dir, os.path.basename(uuid_dll_path))
-
-                shutil.copy(uuid_dll_path, dist_dll_path)
-
-                return ((uuid_dll_path, dist_dll_path, None),)
+                return (
+                    makeDllEntryPoint(
+                        uuid_dll_path, os.path.basename(uuid_dll_path), None
+                    ),
+                )
         elif full_name == "iptc" and getOS() == "Linux":
             import iptc.util  # pylint: disable=I0021,import-error
 
             xtwrapper_dll = iptc.util.find_library("xtwrapper")[0]
             xtwrapper_dll_path = xtwrapper_dll._name  # pylint: disable=protected-access
 
-            dist_dll_path = os.path.join(dist_dir, os.path.basename(xtwrapper_dll_path))
-
-            shutil.copy(xtwrapper_dll_path, dist_dll_path)
-
-            return ((xtwrapper_dll_path, dist_dll_path, None),)
+            return (
+                makeDllEntryPoint(
+                    xtwrapper_dll_path, os.path.basename(xtwrapper_dll_path), None
+                ),
+            )
         elif full_name == "gi._gi":
             gtk_dll_path = locateDLL("gtk-3")
 
@@ -1281,20 +1292,21 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                 gtk_dll_path = locateDLL("gtk-3-0")
 
             if gtk_dll_path is not None:
-                dist_dll_path = os.path.join(dist_dir, os.path.basename(gtk_dll_path))
-                shutil.copy(gtk_dll_path, dist_dll_path)
-
-                return ((gtk_dll_path, dist_dll_path, None),)
-        elif full_name in ("win32api", "pythoncom") and isWin32Windows():
-            # Do this only once, pylint: disable=global-statement
-            global _added_pywin32
+                return (
+                    makeDllEntryPoint(
+                        gtk_dll_path, os.path.basename(gtk_dll_path), None
+                    ),
+                )
 
+        # TODO: This should be its own plugin.
+        elif (
+            full_name in ("win32api", "pythoncom", "win32file", "win32com")
+            and isWin32Windows()
+        ):
             result = []
             pywin_dir = getPyWin32Dir()
 
-            if pywin_dir is not None and not _added_pywin32:
-                _added_pywin32 = True
-
+            if pywin_dir is not None:
                 for dll_name in "pythoncom", "pywintypes":
 
                     pythoncom_filename = "%s%d%d.dll" % (
@@ -1303,12 +1315,13 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                         sys.version_info[1],
                     )
                     pythoncom_dll_path = os.path.join(pywin_dir, pythoncom_filename)
-                    dist_dll_path = os.path.join(dist_dir, pythoncom_filename)
 
                     if os.path.exists(pythoncom_dll_path):
-                        shutil.copy(pythoncom_dll_path, dist_dir)
-
-                        result.append((pythoncom_dll_path, dist_dll_path, None))
+                        result.append(
+                            makeDllEntryPoint(
+                                pythoncom_dll_path, pythoncom_filename, None
+                            )
+                        )
 
             return result
 
@@ -1339,6 +1352,8 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         "pyVmomi.ServerObjects",  # Too large generated code
         "pyglet.gl",  # Too large generated code
         "telethon.tl.types",  # Not performance relevant and slow C compile
+        "importlib_metadata",  # Not performance relevant and slow C compile
+        "comtypes.gen",  # Not performance relevant and slow C compile
     )
 
     def decideCompilation(self, module_name, source_ref):
